// This file is always executed before the App's index.js. It sets up most
// of Pebble Ejecta's functionality and emulates some DOM objects.

// Make 'window' the global scope
self = window = this;
window.top = window.parent = window;

(function(window) {

// The 'ej' object provides some basic info and utility functions
var ej = window.ejecta = new Ejecta.EjectaCore();

// Set up the screen properties and useragent
window.devicePixelRatio = ej.devicePixelRatio;
// these are going to be 0,0 when we're running headless
window.innerWidth = ej.screenWidth;
window.innerHeight = ej.screenHeight;

window.screen = {
	availWidth: window.innerWidth,
	availHeight: window.innerHeight
};

window.navigator = {
	language: ej.language,
	userAgent: ej.userAgent,
	appVersion: ej.appVersion,
	get onLine() { return ej.onLine; }, // re-evaluate on each get
    geolocation: new Ejecta.Location() // replicates geolocation object
};

// The console object
window.console = {
	log: function() {
		var args = Array.prototype.join.call(arguments, ', ');
		ej.log( args );
	},

	assert: function() {
		var args = Array.prototype.slice.call(arguments);
		var assertion = args.shift();
		if( !assertion ) {
			ej.log( 'Assertion failed: ' + args.join(', ') );
		}
	}
};
window.console.debug =
	window.console.info =
	window.console.warn =
	window.console.error =
	window.console.log;

// CommonJS style require()
var loadedModules = {};
window.require = function( name ) {
	var id = name.replace(/\.js$/,'');
	if( !loadedModules[id] ) {
		var exports = {};
		var module = { id: id, uri: id + '.js', exports: exports };
		ejecta.requireModule( id, module, exports );
		// Some modules override module.exports, so use the module.exports reference only after loading the module
		loadedModules[id] = module.exports;
	}

	return loadedModules[id];
};

// Timers
window.performance = {now: function() {return ej.performanceNow();} };
window.setTimeout = function(cb, t){ return ej.setTimeout(cb, t); };
window.setInterval = function(cb, t){ return ej.setInterval(cb, t); };
window.clearTimeout = function(id){ return ej.clearTimeout(id); };
window.clearInterval = function(id){ return ej.clearInterval(id); };
window.requestAnimationFrame = function(cb, element){
	return ej.setTimeout(function(){ cb(ej.performanceNow()); }, 16);
};

// The native Image, Audio, HttpRequest and LocalStorage class mimic the real elements
window.XMLHttpRequest = Ejecta.HttpRequest;
window.Audio = Ejecta.Audio;
window.localStorage = new Ejecta.LocalStorage();
window.WebSocket = Ejecta.WebSocket;
window.Pebble = new Ejecta.Pebble();

var ajax = function(r, successCb, errorCb) {
    var req = new XMLHttpRequest();

    // Open the request:
    var method = r["method"];
    if (!method) {
        method = "GET";
    }
    var url = r["url"];
    req.open(method, url);

    // Add headers:
    var headers = r["headers"];
    for (var key in headers) {
        req.setRequestHeader(key, headers[key]);
    }

    // Add event handlers:
    req.addEventListener("error", function(evt) {
        errorCb("Network error");
    });
    req.addEventListener("abort", function(evt) {
        errorCb("Request cancelled");
    });
    req.addEventListener("load", function(evt) {
        if (req.status / 100 == 2) {
            successCb(req.responseText);
        } else {
            errorCb("" + req.statusText + " (" + req.status + ")");
        }
    });

    // Send it:
    req.send();
};

var ajaxWithUserToken = function(url, method, successCb, errorCb) {
	window.Pebble.getTimelineToken(
		function(token) {
			var headers = {
				'X-User-Token': token
			};
			ajax({url: url, headers: headers, method: method},
				successCb, errorCb);
		},
		errorCb
	);
};

// Timeline subscriptions:

window.Pebble.timelineSubscribe = function(topic, successCb, errorCb) {
	var url = window.Pebble.getTimelineSubscriptionToTopicURL(topic);
	ajaxWithUserToken(url, "POST", successCb,
		function (errString) {
			errorCb("Error subscribing: " + errString);
		});
};

window.Pebble.timelineUnsubscribe = function(topic, successCb, errorCb) {
	var url = window.Pebble.getTimelineSubscriptionToTopicURL(topic);
	ajaxWithUserToken(url, "DELETE", successCb,
		function (errString) {
			errorCb("Error unsubscribing: " + errString);
		});
};

window.Pebble.timelineSubscriptions = function(successCb, errorCb) {
	var url = window.Pebble.getTimelineSubscriptionsListURL();
	ajaxWithUserToken(url, "GET",
		function (response) {
			try {
				var responseJSON = JSON.parse(response);
				successCb(responseJSON["topics"]);
			} catch (err) {
				errorCb("Malformed response from server: " + err);
			}
		},
		function (errString) {
			errorCb("Error fetching subscriptions: " + errString);
		});
}


// Set up a "fake" HTMLElement
HTMLElement = function( tagName ){
	this.tagName = tagName;
	this.children = [];
	this.style = {};
};

HTMLElement.prototype.appendChild = function( element ) {
	this.children.push( element );

	// If the child is a script element, begin to load it
	if( element.tagName == 'script' ) {
		ej.setTimeout( function(){
			ej.include( element.src );
			if( element.onload ) {
				element.onload();
			}
		}, 1);
	}
};

// The document object
window.document = {
	location: { href: 'index' },

	head: new HTMLElement( 'head' ),
	body: new HTMLElement( 'body' ),

	events: {},

	createElement: function( name ) {
    if (typeof name === 'string') {
      switch (name.toLowerCase()) {
        case 'canvas':
        case 'video':
        case 'img':
        case 'input':
        case 'textarea':
          console.log('Not supported when running headlessly: ' + name);
          break;
        case 'audio':
          return new Ejecta.Audio();
      }
    }
		return new HTMLElement( name );
	},

	getElementById: function( id ){
		if( id === 'canvas' ) {
			return window.canvas;
		}
		return null;
	},

	getElementsByTagName: function( tagName ) {
		if( tagName === 'head' ) {
			return [document.head];
		}
		else if( tagName === 'body' ) {
			return [document.body];
		}
		return [];
	},

	addEventListener: function( type, callback, useCapture ){
		if( type == 'DOMContentLoaded' ) {
			ej.setTimeout( callback, 1 );
			return;
		}
		if( !this.events[type] ) {
			this.events[type] = [];

			// call the event initializer, if this is the first time we
			// bind to this event.
			if( typeof(this._eventInitializers[type]) == 'function' ) {
				this._eventInitializers[type]();
			}
		}
		this.events[type].push( callback );
	},

	removeEventListener: function( type, callback ) {
		var listeners = this.events[ type ];
		if( !listeners ) { return; }

		for( var i = listeners.length; i--; ) {
			if( listeners[i] === callback ) {
				listeners.splice(i, 1);
			}
		}
	},

	_eventInitializers: {},
	dispatchEvent: function( event ) {
		var listeners = this.events[ event.type ];
		if( !listeners ) { return; }

		for( var i = 0; i < listeners.length; i++ ) {
			listeners[i]( event );
		}
	}
};

var eventInit = document._eventInitializers;

// DeviceMotion and DeviceOrientation events
// disabled for now...
//var deviceMotion = null;
//var deviceMotionEvent = {
//	type: 'devicemotion',
//	target: canvas,
//	interval: 16,
//	acceleration: {x: 0, y: 0, z: 0},
//	accelerationIncludingGravity: {x: 0, y: 0, z: 0},
//	rotationRate: {alpha: 0, beta: 0, gamma: 0},
//	preventDefault: function(){},
//	stopPropagation: function(){}
//};
//
//var deviceOrientationEvent = {
//	type: 'deviceorientation',
//	target: canvas,
//	alpha: null,
//	beta: null,
//	gamma: null,
//	absolute: true,
//	preventDefault: function(){},
//	stopPropagation: function(){}
//};
//
//eventInit.deviceorientation = eventInit.devicemotion = function() {
//	if( deviceMotion ) { return; }
//
//	deviceMotion = new Ejecta.DeviceMotion();
//	deviceMotionEvent.interval = deviceMotion.interval;
//
//	// Callback for Devices that have a Gyro
//	deviceMotion.ondevicemotion = function( agx, agy, agz, ax, ay, az, rx, ry, rz, ox, oy, oz ) {
//		deviceMotionEvent.accelerationIncludingGravity.x = agx;
//		deviceMotionEvent.accelerationIncludingGravity.y = agy;
//		deviceMotionEvent.accelerationIncludingGravity.z = agz;
//
//		deviceMotionEvent.acceleration.x = ax;
//		deviceMotionEvent.acceleration.y = ay;
//		deviceMotionEvent.acceleration.z = az;
//
//		deviceMotionEvent.rotationRate.alpha = rx;
//		deviceMotionEvent.rotationRate.beta = ry;
//		deviceMotionEvent.rotationRate.gamma = rz;
//
//		document.dispatchEvent( deviceMotionEvent );
//
//
//		deviceOrientationEvent.alpha = ox;
//		deviceOrientationEvent.beta = oy;
//		deviceOrientationEvent.gamma = oz;
//
//		document.dispatchEvent( deviceOrientationEvent );
//	};
//
//	// Callback for Devices that only have an accelerometer
//	deviceMotion.onacceleration = function( agx, agy, agz ) {
//		deviceMotionEvent.accelerationIncludingGravity.x = agx;
//		deviceMotionEvent.accelerationIncludingGravity.y = agy;
//		deviceMotionEvent.accelerationIncludingGravity.z = agz;
//
//		deviceMotionEvent.acceleration = null;
//		deviceMotionEvent.rotationRate = null;
//
//		document.dispatchEvent( deviceMotionEvent );
//	}
//};

// Window events (resize/pagehide/pageshow)

var windowEvents = null;

var lifecycleEvent = {
	type: 'pagehide',
	target: window.document,
	preventDefault: function(){},
	stopPropagation: function(){}
};

var resizeEvent = {
	type: 'resize',
	target: window,
	preventDefault: function(){},
	stopPropagation: function(){}
};

eventInit.pagehide = eventInit.pageshow = eventInit.resize = function() {
	if( windowEvents ) { return; }

	windowEvents = new Ejecta.WindowEvents();

	windowEvents.onpagehide = function() {
		lifecycleEvent.type = 'pagehide';
		document.dispatchEvent( lifecycleEvent );
	};
	windowEvents.onpageshow = function() {
		lifecycleEvent.type = 'pageshow';
		document.dispatchEvent( lifecycleEvent );
	};

	windowEvents.onresize = function() {
		window.innerWidth = ej.screenWidth;
		window.innerHeight = ej.screenHeight;
		document.dispatchEvent(resizeEvent);
	};
};

})(this);
